package de.urszeidler.eclipse.callchain.ui.wizards;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.filesystem.EFS;
import org.eclipse.core.filesystem.IFileStore;
import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.osgi.framework.Bundle;

import de.urszeidler.eclipse.callchain.provider.CallchainEditPlugin;

public abstract class BasicTemplateWizard extends Wizard implements INewWizard {
    private static final String[] KNOWN_ATL_NATURES = {
            "org.atl.eclipse.adt.builder.atlNature",
            "org.eclipse.m2m.atl.adt.builder.atlNature" };
    private static final String[] KNOWN_ATL_BUILDER = {
            "org.atl.eclipse.adt.builder.atlBuilder",
            "org.eclipse.m2m.atl.adt.builder.atlBuilder" };

    private static final String ATL_NATURE = "org.atl.eclipse.adt.builder.atlNature";
    private static final String ATL_BUILDER_ID = "org.atl.eclipse.adt.builder.atlBuilder";
    protected Map<String, String> _variables = new HashMap<String, String>();

    public BasicTemplateWizard() {
        super();
    }

    protected abstract String getModelName();

    protected abstract String getBaseTemplatePath();

    protected abstract IContainer getTargetContainer();
    /**
     * initalise some variables <code>name</code>,<code>container_name</code> and <code>path_name</code>
     * @param name
     * @param cname
     * @param pname
     */
    protected void initVariables(String name, String cname, String pname) {
        _variables = new HashMap<String, String>();
        _variables.put("name", name);

        _variables.put("container_name", cname);
        _variables.put("path_name", pname);

    }

    protected String buildAbsolutePath(String path) {
        if (path.startsWith("/"))
            path = path.substring(1);

        String separator = java.io.File.separator;
        if (separator.equals("\\"))
            path = path.replaceAll("\\\\", "\\\\");

        return path;
    }

    protected void processPath(Enumeration entryPaths, IContainer target,
            Bundle extensionBundle, String basename) throws CoreException {
        for (Enumeration iter = entryPaths; iter.hasMoreElements();) {
            String name = (String) iter.nextElement();
            if (name.endsWith("/.svn/"))
                continue;

            File file = getFile(name);
            IFileStore fileStore = EFS.getLocalFileSystem().getStore(
                    target.getLocationURI());
            IContainer ntarget;
            if (file.isDirectory()) {
                IFileStore dir = fileStore.getChild(
                        name.substring(basename.length())).mkdir(EFS.SHALLOW,
                        null);
                IContainer[] containers = ResourcesPlugin.getWorkspace()
                        .getRoot().findContainersForLocationURI(dir.toURI());
                if (containers.length == 1) {
                    Enumeration entrys = extensionBundle.getEntryPaths(name);
                    ntarget = containers[0];
                    if (entrys != null) {
                        ntarget.refreshLocal(1, null);
                        processPath(entrys, ntarget, extensionBundle, name);
                    }
                }
            }

            processResource(extensionBundle, target, file, name);
        }

    }

    /**
     * @param extensionBundle
     * @param target
     * @param ntarget
     * @param name
     * @return
     * @throws CoreException
     */
    protected void processResource(Bundle extensionBundle, IContainer target,
            File file, String name) throws CoreException {
        // URL templateFileURL = extensionBundle.getEntry(name);
        // extensionBundle.get

        // IFileStore fileStore = EFS.getLocalFileSystem().getStore(
        // target.getLocationURI());
        // if(file.isDirectory()){
        // IFileStore dir =
        // fileStore.getChild(name.substring(BASE_BUNDEL_ENTRY.length())).mkdir(EFS.SHALLOW,
        // null);
        // IContainer[] containers =
        // ResourcesPlugin.getWorkspace().getRoot().findContainersForLocationURI(dir.toURI());
        // if(containers.length==1)
        // ntarget = containers[0];
        //            
        // }else
        generate(new NullProgressMonitor(), file, target);
        // return ntarget;
    }

    /**
     * Returns the file of the given relativePath
     * 
     * @param relativePath
     *            the relativePath of the template which represent the file to
     *            get
     * @return the file
     */
    protected File getFile(String relativePath) {
        try {
            Bundle extensionBundle = Platform.getBundle(WizardsPlugin.getId());
            URL templateFileURL = extensionBundle.getEntry(relativePath);
            FileLocator.toFileURL(templateFileURL);
            return new File(Platform.asLocalURL(templateFileURL).getFile());
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    protected IFile _copyFile(File file, IContainer dest,
            IProgressMonitor monitor) throws CoreException {
        String targetFileName = getProcessedString(file.getName());

        monitor.subTask(targetFileName);
        IFile dstFile = dest.getFile(new Path(targetFileName));

        InputStream stream = null;
        try {
            stream = _getProcessedStream(file, dest.getDefaultCharset());
            if (dstFile.exists()) {
                dstFile.setContents(stream, true, true, monitor);
            } else {
                dstFile.create(stream, true, monitor);
            }

        } catch (IOException ioe) {
            CallchainEditPlugin.log(ioe);
            try {
                if (stream != null) {
                    stream.close();
                }
            } catch (IOException ioe2) {
                CallchainEditPlugin.log(ioe2);
            }
        }

        return dstFile;
    }

    protected InputStream _getProcessedStream(File file, String charset)
            throws IOException {
        FileInputStream stream = new FileInputStream(file);

        InputStreamReader reader = new InputStreamReader(stream);
        int bufsize = 1024;
        char[] cbuffer = new char[bufsize];
        int read = 0;
        StringBuffer keyBuffer = new StringBuffer();
        StringBuffer outBuffer = new StringBuffer();

        boolean replacementMode = false;
        boolean escape = false;
        while (read != -1) {
            read = reader.read(cbuffer);
            for (int i = 0; i < read; i++) {
                char c = cbuffer[i];

                if (escape) {
                    if (c != '%') {
                        outBuffer.append('\\');
                    }

                    outBuffer.append(c);
                    escape = false;
                    continue;
                }

                if (c == '%') {
                    if (replacementMode) {
                        replacementMode = false;
                        String key = keyBuffer.toString();
                        String value = _getVariable(key);
                        outBuffer.append(value);
                        keyBuffer.delete(0, keyBuffer.length());
                    } else {
                        replacementMode = true;
                    }
                } else {
                    if (c == '\\') {
                        escape = true;
                        continue;
                    }

                    if (replacementMode) {
                        keyBuffer.append(c);
                    } else {
                        outBuffer.append(c);
                    }
                }
            }
        }
        stream.close();
        return new ByteArrayInputStream(outBuffer.toString().getBytes(charset));
    }

    protected String _getVariable(String key) {
        if (_variables != null && _variables.get(key) != null) {
            return _variables.get(key);
        }

        return key;
    }

    /**
     * Process the given String through the parameter map
     * 
     * @param source
     *            the source String
     * @return the processed string
     */
    public String getProcessedString(String source) {
        if (source.indexOf('%') == -1) {
            return source;
        }
        int loc = -1;
        StringBuffer buffer = new StringBuffer();
        boolean replacementMode = false;
        for (int i = 0; i < source.length(); i++) {
            char c = source.charAt(i);
            if (c == '%') {
                if (replacementMode) {
                    String key = source.substring(loc, i);
                    String value = _getVariable(key);
                    buffer.append(value);
                    replacementMode = false;
                } else {
                    replacementMode = true;
                    loc = i + 1;
                    continue;
                }
            } else if (!replacementMode) {
                buffer.append(c);
            }
        }
        return buffer.toString();
    }

    /**
     * Generates files as part of the template execution. The default
     * implementation uses template location as a root of the file templates.
     * The files found in the location are processed in the following way: Files
     * and folders are copied directly into the target folder with the
     * conditional generation and variable replacement for files. Variable
     * replacement also includes file names.
     * 
     * @param monitor
     *            progress monitor to use to indicate generation progress
     * @param _source
     * @param _destination
     * @param _destination
     * @return the processed resource
     * @throws CoreException
     */
    public IResource generate(IProgressMonitor monitor, File _source,
            IContainer _destination) throws CoreException {
        monitor.setTaskName("Template Generation Process");
        IResource dest = null;

        dest = _copyFile(_source, _destination, monitor);
        monitor.subTask(""); //$NON-NLS-1$
        monitor.worked(1);
        return dest;
    }

    protected void addATLnature(IProgressMonitor monitor, int natureType) {
        IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(
                getProjectName());
        try {
            IProjectDescription description = project.getDescription();
            if (!description.hasNature(KNOWN_ATL_NATURES[natureType])) {
                String[] natureIds = description.getNatureIds();
                String[] newNatureIds = new String[natureIds.length + 1];
                for (int i = 0; i < natureIds.length; i++) {
                    String nature = natureIds[i];
                    newNatureIds[i] = nature;
                }
                newNatureIds[natureIds.length] = KNOWN_ATL_NATURES[natureType];
                description.setNatureIds(newNatureIds);
            }

            ICommand builderCommand = getProjectBuilderCommand(description,
                    KNOWN_ATL_BUILDER[natureType]);
            if (builderCommand == null) {

                // Add a new build spec
                ICommand command = description.newCommand();
                command.setBuilderName(KNOWN_ATL_BUILDER[natureType]);

                // Commit the spec change into the project
                addProjectBuilderCommand(description, command);
                project.setDescription(description, monitor);
            }

        } catch (CoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }

    protected abstract String getProjectName();

    /**
     * Adds (or updates) a builder in given project description.
     */
    public static void addProjectBuilderCommand(
            IProjectDescription description, ICommand command)
            throws CoreException {
        ICommand[] oldCommands = description.getBuildSpec();
        ICommand oldBuilderCommand = getProjectBuilderCommand(description,
                command.getBuilderName());
        ICommand[] newCommands;
        if (oldBuilderCommand == null) {

            // Add given builder to the end of the builder list
            newCommands = new ICommand[oldCommands.length + 1];
            System
                    .arraycopy(oldCommands, 0, newCommands, 0,
                            oldCommands.length);
            newCommands[oldCommands.length] = command;
        } else {

            // Replace old builder with given new one
            for (int i = 0, max = oldCommands.length; i < max; i++) {
                if (oldCommands[i] == oldBuilderCommand) {
                    oldCommands[i] = command;
                    break;
                }
            }
            newCommands = oldCommands;
        }
        description.setBuildSpec(newCommands);
    }

    /**
     * Returns specified builder from given project description.
     */
    public static ICommand getProjectBuilderCommand(
            IProjectDescription description, String builderID)
            throws CoreException {
        ICommand[] commands = description.getBuildSpec();
        for (int i = commands.length - 1; i >= 0; i--) {
            if (commands[i].getBuilderName().equals(builderID)) {
                return commands[i];
            }
        }
        return null;
    }

    /**
     * @return true if the model file was successfully created
     * @throws CoreException
     * @throws MalformedURLException
     * @generated
     */
    protected boolean createFilesFromTemplate() throws CoreException,
            MalformedURLException {
        Bundle extensionBundle = WizardsPlugin.getDefault().getBundle();
        IContainer target = getTargetContainer();
        Enumeration entryPaths = extensionBundle
                .getEntryPaths(getBaseTemplatePath());

        String apath = buildAbsolutePath(target.getLocationURI().toURL()
                .getPath());
        initVariables(getModelName(), target.getFullPath().toString(), apath);
        processPath(entryPaths, target, extensionBundle, getBaseTemplatePath());

        return true;
    }

}