/**
 * <copyright>
 * </copyright>
 *
 * $Id: CallchainSwitch.java,v 1.2 2008/04/13 20:29:20 urs Exp $
 */
package de.urszeidler.eclipse.callchain.util;

import de.urszeidler.eclipse.callchain.*;

import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

/**
 * <!-- begin-user-doc -->
 * The <b>Switch</b> for the model's inheritance hierarchy.
 * It supports the call {@link #doSwitch(EObject) doSwitch(object)}
 * to invoke the <code>caseXXX</code> method for each class of the model,
 * starting with the actual class of the object
 * and proceeding up the inheritance hierarchy
 * until a non-null result is returned,
 * which is the result of the switch.
 * <!-- end-user-doc -->
 * @see de.urszeidler.eclipse.callchain.CallchainPackage
 * @generated
 */
public class CallchainSwitch<T> {
	/**
	 * The cached model package
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected static CallchainPackage modelPackage;

	/**
	 * Creates an instance of the switch.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public CallchainSwitch() {
		if (modelPackage == null) {
			modelPackage = CallchainPackage.eINSTANCE;
		}
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	public T doSwitch(EObject theEObject) {
		return doSwitch(theEObject.eClass(), theEObject);
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	protected T doSwitch(EClass theEClass, EObject theEObject) {
		if (theEClass.eContainer() == modelPackage) {
			return doSwitch(theEClass.getClassifierID(), theEObject);
		}
		else {
			List<EClass> eSuperTypes = theEClass.getESuperTypes();
			return
				eSuperTypes.isEmpty() ?
					defaultCase(theEObject) :
					doSwitch(eSuperTypes.get(0), theEObject);
		}
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	protected T doSwitch(int classifierID, EObject theEObject) {
		switch (classifierID) {
			case CallchainPackage.MODEL: {
				Model model = (Model)theEObject;
				T result = caseModel(model);
				if (result == null) result = caseArtifact(model);
				if (result == null) result = caseDocumentable(model);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.ATL_RES: {
				AtlRes atlRes = (AtlRes)theEObject;
				T result = caseAtlRes(atlRes);
				if (result == null) result = caseArtifact(atlRes);
				if (result == null) result = caseDocumentable(atlRes);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.CALL: {
				Call call = (Call)theEObject;
				T result = caseCall(call);
				if (result == null) result = caseDocumentable(call);
				if (result == null) result = caseCallable(call);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.MM: {
				MM mm = (MM)theEObject;
				T result = caseMM(mm);
				if (result == null) result = caseArtifact(mm);
				if (result == null) result = caseDocumentable(mm);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.CALLS: {
				Calls calls = (Calls)theEObject;
				T result = caseCalls(calls);
				if (result == null) result = caseDocumentable(calls);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.ATL_LIBARY: {
				AtlLibary atlLibary = (AtlLibary)theEObject;
				T result = caseAtlLibary(atlLibary);
				if (result == null) result = caseArtifact(atlLibary);
				if (result == null) result = caseDocumentable(atlLibary);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.MODEL_ALIAS: {
				ModelAlias modelAlias = (ModelAlias)theEObject;
				T result = caseModelAlias(modelAlias);
				if (result == null) result = caseModel(modelAlias);
				if (result == null) result = caseArtifact(modelAlias);
				if (result == null) result = caseDocumentable(modelAlias);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.GENERATOR: {
				Generator generator = (Generator)theEObject;
				T result = caseGenerator(generator);
				if (result == null) result = caseDocumentable(generator);
				if (result == null) result = caseCallable(generator);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.ARTIFACT: {
				Artifact artifact = (Artifact)theEObject;
				T result = caseArtifact(artifact);
				if (result == null) result = caseDocumentable(artifact);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.EMF_GENERATOR: {
				EMF_Generator emF_Generator = (EMF_Generator)theEObject;
				T result = caseEMF_Generator(emF_Generator);
				if (result == null) result = caseGenerator(emF_Generator);
				if (result == null) result = caseDocumentable(emF_Generator);
				if (result == null) result = caseCallable(emF_Generator);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.USEIT_GENERATOR: {
				Useit_Generator useit_Generator = (Useit_Generator)theEObject;
				T result = caseUseit_Generator(useit_Generator);
				if (result == null) result = caseGenerator(useit_Generator);
				if (result == null) result = caseDocumentable(useit_Generator);
				if (result == null) result = caseCallable(useit_Generator);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.TOPCASED_GENERATOR: {
				TopcasedGenerator topcasedGenerator = (TopcasedGenerator)theEObject;
				T result = caseTopcasedGenerator(topcasedGenerator);
				if (result == null) result = caseGenerator(topcasedGenerator);
				if (result == null) result = caseDocumentable(topcasedGenerator);
				if (result == null) result = caseCallable(topcasedGenerator);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.MM_ALIAS: {
				MMAlias mmAlias = (MMAlias)theEObject;
				T result = caseMMAlias(mmAlias);
				if (result == null) result = caseMM(mmAlias);
				if (result == null) result = caseArtifact(mmAlias);
				if (result == null) result = caseDocumentable(mmAlias);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.DOCUMENTABLE: {
				Documentable documentable = (Documentable)theEObject;
				T result = caseDocumentable(documentable);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.COMMENT: {
				Comment comment = (Comment)theEObject;
				T result = caseComment(comment);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.AM3_GENERATOR: {
				AM3_Generator am3_Generator = (AM3_Generator)theEObject;
				T result = caseAM3_Generator(am3_Generator);
				if (result == null) result = caseGenerator(am3_Generator);
				if (result == null) result = caseDocumentable(am3_Generator);
				if (result == null) result = caseCallable(am3_Generator);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.CALLABLE: {
				Callable callable = (Callable)theEObject;
				T result = caseCallable(callable);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.GMF_GENERATOR: {
				GMF_Generator gmF_Generator = (GMF_Generator)theEObject;
				T result = caseGMF_Generator(gmF_Generator);
				if (result == null) result = caseGenerator(gmF_Generator);
				if (result == null) result = caseDocumentable(gmF_Generator);
				if (result == null) result = caseCallable(gmF_Generator);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CallchainPackage.GENERIC_GENERATOR: {
				Generic_Generator generic_Generator = (Generic_Generator)theEObject;
				T result = caseGeneric_Generator(generic_Generator);
				if (result == null) result = caseGenerator(generic_Generator);
				if (result == null) result = caseDocumentable(generic_Generator);
				if (result == null) result = caseCallable(generic_Generator);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			default: return defaultCase(theEObject);
		}
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Model</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Model</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseModel(Model object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Atl Res</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Atl Res</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseAtlRes(AtlRes object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Call</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Call</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseCall(Call object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>MM</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>MM</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseMM(MM object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Calls</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Calls</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseCalls(Calls object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Atl Libary</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Atl Libary</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseAtlLibary(AtlLibary object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Model Alias</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Model Alias</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseModelAlias(ModelAlias object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Generator</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Generator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseGenerator(Generator object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Artifact</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Artifact</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseArtifact(Artifact object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>EMF Generator</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>EMF Generator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseEMF_Generator(EMF_Generator object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Useit Generator</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Useit Generator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseUseit_Generator(Useit_Generator object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Topcased Generator</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Topcased Generator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseTopcasedGenerator(TopcasedGenerator object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>MM Alias</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>MM Alias</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseMMAlias(MMAlias object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Documentable</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Documentable</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDocumentable(Documentable object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Comment</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Comment</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseComment(Comment object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>AM3 Generator</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>AM3 Generator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseAM3_Generator(AM3_Generator object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Callable</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Callable</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseCallable(Callable object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>GMF Generator</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>GMF Generator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseGMF_Generator(GMF_Generator object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Generic Generator</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Generic Generator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseGeneric_Generator(Generic_Generator object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>EObject</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch, but this is the last case anyway.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>EObject</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject)
	 * @generated
	 */
	public T defaultCase(EObject object) {
		return null;
	}

} //CallchainSwitch
